﻿#include "IocpWorker.h"
#include "Session.h"
#include "IocpInterface.h"
#include "Packet.h"

/**
@brief 생성자. 
@param hIOCP GQCS에서 사용할 IOCP핸들
*/
CIocpWorker::CIocpWorker(HANDLE hIOCP) : m_hIOCP(hIOCP)
{
	Start();
}

/**
@brief 소멸자. 스레드를 멈춘다.
*/
CIocpWorker::~CIocpWorker()
{
	Stop();
}

/**
@brief IOCP 루프 메소드
@desc GQCS와 여러 IO동작을 한다.
*/
void CIocpWorker::Act()
{
	BOOL bRet;
	CSession* pcSession = NULL;
	CIocpContext* pIO = NULL;
	CIocpInterface* pcWorker = NULL;
	DWORD dwTransfer;
	while(1)
	{
		bRet = GetQueuedCompletionStatus(m_hIOCP, &dwTransfer, (LPDWORD)&pcSession, (LPOVERLAPPED*)&pIO, INFINITE);
		if(!bRet || (bRet && dwTransfer == 0)) 
		{
			if(pIO)
			{
				if(pIO->GetIOType() == IOTYPE_ACCEPT) //Accept 또는 Connect 신호 온 것!
				{
					LOG_DEBUG(_T("%s [%d] [%d] [%d] [%p, %d]"), __TFUNCTION__, WSAGetLastError(), bRet, dwTransfer, pIO, pIO->GetIOType());
					pcSession = pIO->GetSession();
					if(bRet)
					{
						pcSession->OnConnect();
					}
					else
					{
						new CPacket(PACKETEVENT_DISCONNECT, pcSession);
					}
				}
				else if(pIO->GetIOType() == IOTYPE_REFRESH)
				{
					pcSession->OnRefresh();
				}
				else//연결 끊긴겨~
				{
					LOG_DEBUG(_T("%s Close Reason [%d] [%d] [%d] [%p, %d]"), __TFUNCTION__, WSAGetLastError(), bRet, dwTransfer, pIO, pIO->GetIOType());
					new CPacket(PACKETEVENT_DISCONNECT, pcSession);
				}
				continue;
			}
			else
			{				
				LOG_INFO(_T("%s pIO == NULL"), __TFUNCTION__);
				Stop();
				return;
			}
		}

		//pIO가 NULL이면 IO완료 패킷이 아니라 사용자 메시지다
		if(pcSession->isConnected())
		{
			if(pIO->GetIOType() == IOTYPE_RECV)
			{
				pcSession->OnReceive(dwTransfer, pIO->GetHeadBuffer());
			}
			else if(pIO->GetIOType() == IOTYPE_SEND)
			{
				pcSession->OnSend();
			}
		}
	}
}

/**
@brief 스레드를 멈춘다. 그리고 IOCP큐도 종료 메시지를 보낸다.
*/
void CIocpWorker::Stop()
{
	__super::Stop();
	PostQueuedCompletionStatus(m_hIOCP, 0, NULL, NULL);
}