
#include "Pipe.h"

CPipe::CPipe(const tstring& strName, CPeer* pcPeer) : CIoInterface(pcPeer), m_strName(strName), m_hPipe(INVALID_HANDLE_VALUE), m_bTryConnect(false), m_bConnected(false), 
m_cPipeContextAccept(this, IOTYPE_ACCEPT), m_cPipeContextRecv(this, IOTYPE_RECV), m_cPipeContextSend(this, IOTYPE_SEND),
m_pcSendBuffer(new CBuffer(PIPE_BUF_SIZE)), m_pcRecvBuffer(new CBuffer(PIPE_BUF_SIZE))
{
	
}

CPipe::~CPipe()
{
	
}

bool CPipe::Receive()
{
	if(!isConnected())
		return false;

	DWORD nRecv = 0;
	if(!ReadFile(m_hPipe, m_pcRecvBuffer->GetHeadBuffer(), m_pcRecvBuffer->GetRemainSize(), &nRecv, GetPipeContextRecv()))
	{
		if(ERROR_IO_PENDING != GetLastError())
		{
			LOG_INFO(_T("%s ReadFile Failed [%d] [%p, %d]"), __TFUNCTION__, GetLastError(), m_pcRecvBuffer->GetHeadBuffer(), m_pcRecvBuffer->GetRemainSize());
			return false;
		}
	}
	return true;
}

bool CPipe::Send(const unsigned int nLen, const void* pData, const unsigned int nAddLen /* = 0 */, const void* pAddData /* = 0 */)
{
	if(!isConnected())
		return false;

	if(!m_pcSendBuffer->AddData(nLen, pData, nAddLen, pAddData))
	{
		LOG_INFO(_T("%s m_pcSendBuffer->AddData(%d, %p, %d, %p) Failed [%d]"), __TFUNCTION__, nLen, pData, nAddLen, pAddData, GetLastError());
		return false;
	}

	DWORD nWritten = 0;
	if(!WriteFile(m_hPipe, m_pcSendBuffer->GetHeadBuffer(), m_pcSendBuffer->GetUsingSize(), &nWritten, GetPipeContextSend()))
	{
		if(ERROR_IO_PENDING != GetLastError())
		{
			LOG_INFO(_T("%s WriteFile(%d, %d, %d) Failed [%d]"), __TFUNCTION__, m_hPipe, m_pcSendBuffer->GetHeadBuffer(), m_pcSendBuffer->GetUsingSize(), GetLastError());
			return false;
		}
	}
	return true;
}

void CPipe::Register()
{
	GetPipeContextAccept()->Register();
	GetPipeContextRecv()->Register();
	GetPipeContextSend()->Register();
}

bool CPipe::OnReceive(unsigned int nLen, void* data)
{
	m_pcRecvBuffer->JumpTail(nLen);
	while(0 < m_pcRecvBuffer->GetUsingSize())
	{
		int nPacketLen = m_pcPeer->PacketParse(m_pcRecvBuffer->GetUsingSize(), m_pcRecvBuffer->GetHeadBuffer());
		if(nPacketLen == 0)
			break;

		m_pcPeer->OnReceive(nPacketLen, m_pcRecvBuffer->GetHeadBuffer());
		m_pcRecvBuffer->JumpHead(nPacketLen);
	}
	m_pcRecvBuffer->Compress();
	Receive();
	return true;
}

void CPipe::OnConnect()
{
	m_pcPeer->OnConnect();
	m_bConnected = true;
	Receive();
}

void CPipe::OnDisconnect()
{
	m_pcPeer->OnDisconnect();
	m_pcRecvBuffer->Init();
	m_pcSendBuffer->Init();
	m_bConnected = false;
}

void CPipe::OnSend(unsigned int nLen)
{
	m_pcSendBuffer->JumpHead(nLen);
	m_pcSendBuffer->Compress();
}