
#include "PipeContext.h"
#include "Pipe.h"

CPipeContext::CPipeContext(CPipe* pcPipe, IO_TYPE eIoType) : m_pcPipe(pcPipe), m_eIoType(eIoType)
{
	this->hEvent = CreateEvent(FALSE, FALSE, NULL, NULL);
}

CPipeContext::~CPipeContext()
{
	if(GetSmartSingleton()->UnRegister(this))
	{
		//
	}
	SAFE_CLOSE_HANDLE(this->hEvent);
}

void CPipeContext::Register()
{
	GetSmartSingleton()->Register(this);
}

void CPipeContext::Proc()
{
	DWORD dwTransfer;
	BOOL bSuccess = GetOverlappedResult( 
		m_pcPipe->GetPipe(), // handle to pipe 
		this, // OVERLAPPED structure 
		&dwTransfer,            // bytes transferred 
		FALSE);            // do not wait 

	if(bSuccess == FALSE)
	{
		m_pcPipe->OnDisconnect();
		return;
	}

	if(bSuccess == TRUE && dwTransfer == 0)
	{
		if(GetIOType() == IOTYPE_ACCEPT)
		{
			m_pcPipe->OnConnect();
			return;
		}

		m_pcPipe->OnDisconnect();
		return;
	}

	switch(GetIOType())
	{
	case IOTYPE_RECV:
		{
			m_pcPipe->OnReceive(dwTransfer, m_pcPipe->GetRecvHeadBuffer());
		}
		break;
	case IOTYPE_SEND:
		{
			m_pcPipe->OnSend(dwTransfer);
		}
		break;
	default: 
		{
			LOG_ERROR(_T("Invalid pipe state. %d"), GetIOType()); 
			m_pcPipe->Disconnect();
			return;
		}
	}
}