#pragma once

#include "Common.h"
#include "PipeContext.h"
#include "Peer.h"
#include "AutoPtr.h"
#include "IoInterface.h"
#include "Buffer.h"

const int PIPE_BUF_SIZE = 1024;

class CPipe : public CIoInterface
{
	friend class CPipeContext;
public:
	CPipe(const tstring& strName, CPeer* pcPeer = new CPeer);
	~CPipe();

	virtual void Disconnect() = 0;

public:
	virtual void Act() = 0;

	/// 
	virtual bool Send(const unsigned int nLen, const void* pData, const unsigned int nAddLen = 0, const void* pAddData = 0);

	const HANDLE& GetPipe(){return m_hPipe;}

	char* GetRecvHeadBuffer(){return m_pcRecvBuffer->GetHeadBuffer();}

	bool isConnected(){return m_bConnected;}

protected:
	void SetPipe(HANDLE& hPipe){m_hPipe = hPipe;}

	const tstring& GetName(){return m_strName;}

	CPipeContext* GetPipeContextAccept(){return &m_cPipeContextAccept;}
	CPipeContext* GetPipeContextRecv(){return &m_cPipeContextRecv;}
	CPipeContext* GetPipeContextSend(){return &m_cPipeContextSend;}

	bool isTryConnect(){return m_bTryConnect;}
	void SetTryConnect(bool bTryConnect){m_bTryConnect = bTryConnect;}

	void Register();

	bool Receive();

	virtual void OnConnect();
private:
	/// 
	virtual bool OnReceive(unsigned int nLen, void* data);

	virtual void OnSend(unsigned int nLen);

	virtual void OnDisconnect();

private:

	bool m_bConnected;

	HANDLE m_hPipe;

	tstring m_strName;

	CPipeContext m_cPipeContextAccept;
	CPipeContext m_cPipeContextRecv;
	CPipeContext m_cPipeContextSend;

	ELKY::CAutoPtr<CBuffer> m_pcSendBuffer;
	ELKY::CAutoPtr<CBuffer> m_pcRecvBuffer;

	bool m_bTryConnect;
};