﻿#pragma once

#include "Event.h"

///Thread에 사용될 함수의 원형
inline DWORD WINAPI ThreadFunc(void* lpVoid);

/**
@brief Thread를 편리하게 사용하도록 랩핑한 클래스.
*/
class CThread
{
public:
	/**
	@brief 소멸자. 쓰레드를 종료 한다.
	*/
	virtual ~CThread()
	{
		Release();
	}

	/**
	@brief Active상태 동안 Act메소드를 반복 실행한다.
	*/
	inline void Run()
	{
		while(m_bActive)
		{
			if(m_bRunning)
			{
				Act();
			}
		}
	}

	/**
	@brief 스레드 릴리즈
	*/
	virtual void Release()
	{
		if(INVALID_HANDLE_VALUE != m_hThread)
		{
			Stop();
			WaitForSingleObject(m_hThread, INFINITE);
			SAFE_CLOSE_HANDLE(m_hThread);
		}
	}

	/**
	@brief 스레드 정지
	*/
	virtual void Stop()
	{
		InterlockedExchange((LONG *)(&m_bActive), (LONG)FALSE);
		InterlockedExchange((LONG *)(&m_bRunning), (LONG)FALSE);
	}

	/**
	@brief 스레드 시작
	*/
	void Start()
	{
		InterlockedExchange((LONG *)(&m_bActive), (LONG)TRUE);
		InterlockedExchange((LONG *)(&m_bRunning), (LONG)TRUE);
		m_hThread = CreateThread(NULL, 0, ::ThreadFunc, this, 0, NULL);
	}

	/**
	@brief 스레드 멈춤
	*/
	virtual void Pause()
	{
		InterlockedExchange((LONG *)(&m_bRunning), (LONG)FALSE);
	}

	/**
	@brief 스레드 재시작
	*/
	void Resume()
	{
		InterlockedExchange((LONG *)(&m_bRunning), (LONG)TRUE);
	}

	///반복적으로 불려지는 메소드. 상속 받은 클래스에서 재정의해서 사용해야함.
	virtual void Act() = 0;


protected:
	/**
	@brief 생성자. 쓰레드를 생성한다. 상속으로만 생성할 수 있도록 protected로 설정했다.
	*/
	CThread() : m_bActive(FALSE), m_bRunning(FALSE), m_hThread(INVALID_HANDLE_VALUE)
	{
		
	}

private:
	///활성화 여부
	BOOL m_bActive;

	///동작 여부
	BOOL m_bRunning;

	///쓰레드 핸들
	HANDLE m_hThread;
};

/**
@brief Thread함수. CThread 클래스의 Run 메소드를 호출해준다.
*/
inline DWORD WINAPI ThreadFunc(void* lpVoid)
{
	((CThread*)lpVoid)->Run();
	return true;
}
