﻿#pragma once

#ifdef _WIN64
#error Win64 is not supported
#endif

#include "debug_stream.h"
#include "dbghelp.h"

#pragma warning (disable: 4311 4312)

class sym_engine
{
 public:
	sym_engine (unsigned);
	~sym_engine();

	void address(unsigned a)		{ m_address = a; }
	unsigned address(void) const	{ return m_address; }
	
	// symbol handler queries
	unsigned module  (char *, unsigned);
	unsigned symbol  (char *, unsigned, unsigned * = 0);
	unsigned fileline(char *, unsigned, unsigned *, unsigned * = 0);

	// stack walk
	BOOL stack_first (CONTEXT* pctx);
	BOOL stack_next  ();

	/*  format argument
		%m  : module
		%f  : file
		%l  : line
		%ld : line's displacement
		%s  : symbol
		%sd : symbol's displacement
		%a	: function address
		
		for example	
		"%f(%l) : %m at %s\n"
		"%m.%s + %sd bytes, in %f:line %l\n"
	*/	
	static BOOL stack_trace(debugstream&, CONTEXT *, unsigned skip = 0, const char * fmt = default_fmt());
	static BOOL stack_trace(debugstream&, sym_engine&, CONTEXT *,
			unsigned skip = 1, const char * fmt = default_fmt());	
 private:
	static const char * default_fmt() { return "******%f(%l) : %m\r\n%a %s\r\n"; }	
	static BOOL get_line_from_addr (HANDLE, unsigned, unsigned *, IMAGEHLP_LINE *);
	static unsigned get_module_basename (HMODULE, char *, unsigned);
	
	BOOL check();

 private:
	unsigned		m_address;
	DWORD			m_Params[4];
	BOOL			m_ok;
	STACKFRAME *	m_pframe;
	CONTEXT *		m_pctx;
	
 private:
	class guard
	{	
	 private:
		guard();
	 public:
		~guard();
		BOOL init();		
		BOOL fail() const { return m_ref == -1; }
		static guard & instance() 
		{
			static guard g; 
			return g;
		}
	private:		
		void clear();
		BOOL load_module(HANDLE, HMODULE);
		int  m_ref;
	};
};